//
//  Shaders.metal
//  KDE
//
//  Created by Ronja Enseleit on 01.07.23.
//

// File for Metal kernel and shader functions

#include <metal_stdlib>
#include <simd/simd.h>

// Including header shared between this Metal shader code and Swift/C code executing Metal API commands
#import "ShaderTypes.h"

using namespace metal;

typedef struct
{
    float2 dstP0 [[attribute(VertexAttributeDstP0)]];
    float2 dstP1 [[attribute(VertexAttributeDstP1)]];
    float2 srcP0 [[attribute(VertexAttributeSrcP0)]];
    float2 srcP1 [[attribute(VertexAttributeSrcP1)]];
    float4 color [[attribute(VertexAttributeColor)]];
    int expandR [[attribute(VertexAttributeExpandR)]];
} Vertex;

typedef struct
{
    float4 position [[position]];
    float4 color;
    float2 texCoord;
    int expandR;
} ColorInOut;

constant float2 vertices[6] = {
    {-1.0, -1.0},
    {+1.0, -1.0},
    {-1.0, +1.0},
    {+1.0, -1.0},
    {+1.0, +1.0},
    {-1.0, +1.0}
};

vertex ColorInOut vertexShader(Vertex in [[stage_in]],
                               constant Uniforms & uniforms [[ buffer(BufferIndexUniforms) ]],
                               uint vertex_id [[vertex_id]])
{
    ColorInOut out;
    
    float2 v = vertices[vertex_id];
    
    float2 dstHalfSize = (in.dstP1 - in.dstP0) / 2.0;
    float2 dstCenter = (in.dstP0 + in.dstP1) / 2.0;
    float2 dstPos = v * dstHalfSize + dstCenter;
    //dstPos.y = uniforms.resolution.y - dstPos.y;
    out.position = float4(2.0 * (dstPos.x / uniforms.resolution.x) - 1.0,
                          2.0 * (dstPos.y / uniforms.resolution.y) - 1.0,
                          0.0,
                          1.0);
    out.color = in.color;
    out.position.y *= -1.0;
    
    float2 srcHalfSize = (in.srcP1 - in.srcP0) / 2.0;
    float2 srcCenter = (in.srcP0 + in.srcP1) / 2.0;
    float2 srcPos = v * srcHalfSize + srcCenter;
    out.texCoord = srcPos;
    out.expandR = in.expandR;
    
    return out;
}

fragment float4 fragmentShader(ColorInOut in [[stage_in]],
                               constant Uniforms & uniforms [[ buffer(BufferIndexUniforms) ]],
                               texture2d<uint> colorMap     [[ texture(TextureIndexColor) ]])
{
    constexpr sampler colorSampler(mip_filter::linear,
                                   mag_filter::linear,
                                   min_filter::linear);

    float4 colorSample = float4(colorMap.sample(colorSampler, in.texCoord.xy)) / 255.0 * in.color;

    if (in.expandR == 1)
        colorSample = colorSample.rrrr;
    
    return colorSample;
}
